/* the main routine for adore2 */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last Change : 7/16/87 */

/* 
 * EE244 Project.
 * Modifications to Adore
 * Henry Chang
 * henryc@ic.Berkeley.EDU
 * Fall, 1990
 */
/*
 * Last update: Jan 1992 - Enrico Malavasi (malavasi@ic.berkeley.edu)
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>
#include "errtrap.h"
#include "utility.h"
#include "options.h"
#include "ADglobals.h"

#define ADORE_RELEASE "2.42"
#define ADORE_DATE    "June 19, 1992"

#define C_FILE_NAME         "ADmain.c" 
#define TEMP_DIR_DEFAULT    "/tmp/adore.tmp"
#define TEKFILE_DEFAULT     "tekfile"
#define CKTOUT_TMP_FILE     "_cktout"
#define INSTANCES_DEFAULT   "instances"
#define LIB_DEFAULT         "lib"

/* Global Variables */
int Numinteg;        /* total number of opamps (integrator blocks) */
int Numcblk;         /* total number of capacitor blocks */
int Numsw;           /* total number of switches */
int Numclk;          /* number of clocks */
int Numcap;          /* total number of capacitors */
int Numnet;          /* total number of nets including the ground */
int Numterm;         /* total number of terminals including ground terminals */

struct _module *modlist[NUMMODTYPES];     /* list of modules for each type of
                                                OPAMP, CBLK, SWCH, etc.  */
struct _net *netlist;                     /* list of nets */
struct _net **ordered_nets;               /* list of nets ordered according
                                                to their number*/
struct _lib *liblist;                     /* list of library entries */
struct _channel *chanlist[MAXNUMCH];      /* list of channels */

char *cktname = "SHOULDNT_APPEAR";

int gerror;                               /* global error flag */

int hcc_debug_flg = 0;				/* Henry Chang's Debug Flag */
int verbose_flg = 0;				/* Flag for verbose mode */

optionStruct optionList[] = {
    { "h",      "",
        "Prints this message."},
    { "v", 	"",	
	"verbose, program progress will be printed"},
    { "C",      "",
        "Clean up, deletes all temporary directories only"},
    { "D",	"",
        "Debug on, debug messages will be printed, sets hcc_debug_flg=1"},
    { "T",	"", 
	"Keep Temporary Directory upon successful completion"},
    { "o",	"cell",		
	"output cell name (defaults to 'circuit_file'.oct)"},
    { "t",	"tech_file",	
	"Filename of file containing technology (defaults to \"tekfile\")" },
    { "I",	"instances_dir",
        "Directory name for generated instances \n(defaults to \"'circuit_file'.instances\")" },
    { "L",	"library_dir",
        "Directory name of leaf cells (defaults to \"lib\")" },
    { "c", "",                  "Don't route the clock lines" },
    { OPT_RARG,	"circuit_file",	"Filename of file containing circuit" },
    { 0, 0, 0 }
};

static void
  ADoptions (argc,argv,cktfile,tekfile,outputcell,instances,lib,T_flg,C_flg, noClockLines) 
int argc;
char **argv;
char **cktfile;
char **tekfile;
char **outputcell;
char **instances;
char *lib;
int *T_flg;
int *C_flg;
int *noClockLines;
{
    int option;		/* option letter from optGetOption */
    char *check;

    errProgramName (argv[0]);

    printf("This is Adore - Switch-Capacitor Layout Generator\n");
    printf("Rel.%s - %s\n", ADORE_RELEASE, ADORE_DATE);
    strcpy( lib, LIB_DEFAULT );
    *tekfile = TEKFILE_DEFAULT;

    while ( ( option = optGetOption (argc,argv)) != EOF) {
        switch (option) {
	  case 'c': (*noClockLines) = 1;  break;
	  case 'o':
	    *outputcell = util_strsav(optarg);
	    break;
	  case 't': 
	    *tekfile = util_strsav(optarg);
	    break;
	  case 'v':
	    verbose_flg = 1;
	    break;
	  case 'C':
	    *C_flg = 1;
	    break;
	  case 'D':
	    hcc_debug_flg = 1;
	    break;
	  case 'I':
	    *instances = util_strsav(optarg);
	    break;
	  case 'L':
	    strcpy (lib,optarg);
	    break;
	  case 'T':
	    *T_flg = 1;
	    break;
	  default:
	    optUsage ();
        }
    }
    
    if (!(*C_flg )) {
	if (optind == argc)  optUsage ();
	*cktfile = util_strsav(argv[optind]);
    }

    if(*outputcell == NULL)  {
	*outputcell = (char *) calloc(strlen(*cktfile)+5, sizeof(char));
	sprintf (*outputcell, "%s.oct", *cktfile);
    }
    if(*instances == NULL) {
	*instances = (char *) calloc(strlen(*cktfile)+strlen(INSTANCES_DEFAULT)+2, sizeof(char));
	sprintf (*instances, "%s.%s",*cktfile, INSTANCES_DEFAULT); 
    }
    
    VERBOSE( printf ("Circuit File: %s\n",*cktfile));
    VERBOSE( printf ("Technology File: %s\n",*tekfile));
    VERBOSE( printf ("Output Cell name: %s\n",*outputcell));
    VERBOSE( printf ("Instances directory: %s\n",*instances));
    VERBOSE( printf ("Library directory: %s\n",lib));
    
    /*
     * Instances directory cannot have path information
     * Problem is in ADScreateRow which needs to know about the
     * structure of instances.  It will now assume it is one
     * directory underneath the current working directory.
     * -hcc
     */

    for (check=(*instances); *check!=0; check++) {
       if (*check == '/') {
       errRaise (C_FILE_NAME,0,
		 "The instances directory must reside in the current working directory.");
       }   
    }
}

static void
  ADinit (argv,cktfile,tekfile,instances,cktoutfile,tmp_dir)
char **argv;
char *cktfile;
char *tekfile;
char *instances;
char **cktoutfile;
char **tmp_dir;
{
    char unix_command [UNIX_COMMAND_SIZE];
    struct stat buf;	                /* Used for stat command */
    
    /* Checking for existence of cktfile and tekfile */
    if ( stat(cktfile,&buf) != 0 ) {
	errRaise (C_FILE_NAME,0,"Circuit File: %s, does not exist.", cktfile);
    }
    if ( stat(tekfile,&buf) != 0 ) {
	errRaise (C_FILE_NAME,0,"Technology File: %s, does not exist.", tekfile);
    }
    /* Checking on the existence of the instances directory */
    if ( stat(instances,&buf) == 0 ) {
	if ((buf.st_mode & S_IFDIR) != S_IFDIR) {
	    errRaise (C_FILE_NAME,0,"%s is not a directory.",instances);
	} /* if */
    }
    else {
	VERBOSE (printf ("Creating instances directory: %s\n",instances));
	sprintf (unix_command, "mkdir %s", instances);
	SYSTEM (unix_command);
    } /* else, directory does not exist */
    
    /* Creating Temporary directory */
    *tmp_dir = (char *) calloc(strlen("/tmp/adore.tmp.")+15, sizeof(char));
    sprintf (*tmp_dir,"/tmp/adore.tmp.%d", (int) getpid());
    *cktoutfile = (char *) calloc(strlen(*tmp_dir)+strlen(CKTOUT_TMP_FILE)+2, sizeof(char));
    sprintf(*cktoutfile, "%s/%s", *tmp_dir, CKTOUT_TMP_FILE);
    VERBOSE (printf ("Temporary Directory Name: %s\n",*tmp_dir));
    VERBOSE (printf ("Circuit Ouput Temporary File: %s\n",*cktoutfile));
    if ( stat(*tmp_dir,&buf) == 0 ) {
	VERBOSE (printf("Removing directory %s\n",*tmp_dir));
	sprintf (unix_command, "rm -rf %s", *tmp_dir);
	SYSTEM (unix_command);
    }
    VERBOSE (printf("Creating directory %s\n",*tmp_dir));
    sprintf (unix_command, "mkdir %s", *tmp_dir);
    SYSTEM (unix_command);
}

static void
  ADopen (cktfile, tekfile, cktoutfile, fpckt, fptek, fpout)
char *cktfile, *tekfile, *cktoutfile;
FILE **fpckt, **fptek, **fpout;
{
  
    if (( *fpckt = fopen (cktfile,"r")) == NULL ) {
	errRaise (C_FILE_NAME,0,"Can't open %s.",cktfile);
    } 
    if (( *fptek = fopen (tekfile,"r")) == NULL ) {
	errRaise (C_FILE_NAME,0,"Can't open %s.",tekfile);
    } 
    if (( *fpout = fopen (cktoutfile,"w")) == NULL ) {
	errRaise (C_FILE_NAME,0,"Can't open %s.",cktoutfile);
    } 
}

static void
  ADclose (fpckt, fptek, fpout, tmp_dir,T_flg)
FILE *fpckt, *fptek, *fpout;
char *tmp_dir;
int T_flg;
{
    char unix_command [UNIX_COMMAND_SIZE];

    if( fpckt )   fclose( fpckt );
    if( fptek )   fclose( fptek );
    if( fpout )   fclose( fpout );

    if ( T_flg ) {
	VERBOSE( printf("Temporary directory is %s\n", tmp_dir) );
    } else {
	VERBOSE( printf("Removing temporary directory %s\n", tmp_dir) );
	sprintf (unix_command, "rm -rf %s", tmp_dir);
	SYSTEM (unix_command);
    }
}

/* 
 * Hack, something wrong with create-octtools-makefile
 * need VOVdisable.
 */

VOVdisable()
{
}


main(argc, argv)
    int argc;
    char **argv;
{
    int counter;
    char unix_command[UNIX_COMMAND_SIZE];
    
    struct _module *tmpmod;
    struct _cblk *tmpcblk;
    
    FILE *fpckt = NULL;		       	/* File pointer to cktfile 	   */
    FILE *fptek = NULL;			/* File pointer to tekfile	   */
    FILE *fpout = NULL;			/* File pointer to cktout_tmp_file */
    int T_flg = 0;			/* -T flag			   */
    int C_flg = 0;			/* -C flag                         */
    int noClockLines = 0;
    char *cktfile = NULL;	        /* Input Circuit File Name         */
    char *tekfile = NULL;	        /* Input Technology File Name      */
    char *cktoutfile = NULL;	        /* cktout_tmp File Name 	   */
    char *outputcell = NULL;      	/* Output Cell Name, OCT           */
    char *tmp_dir    = NULL;     	/* Name of Temporary directory     */
    char *instances  = NULL;     	/* directory for instances         */
    char lib[FILENAME_SIZE];		/* library directory	           */
    
    ADoptions (argc,argv,
	       &cktfile,
	       &tekfile,
	       &outputcell,
	       &instances, lib,
	       &T_flg,&C_flg,&noClockLines);
    
    if( C_flg==1) {
	sprintf (unix_command, "rm -r %s.*", TEMP_DIR_DEFAULT);
	SYSTEM (unix_command);
	exit( 0 );
    }
    
    ADinit (argv,cktfile,tekfile,instances, &cktoutfile, &tmp_dir);
    ADopen (cktfile, tekfile, cktoutfile, &fpckt, &fptek, &fpout);
    
    /* read the circuit file and set up the data structures */
    VERBOSE (printf("Processing the circuit file...\n")); 
    INPsetupAll(fpckt, fpout, lib);
    
    VERBOSE (printf("Processing the technology information \n"));
    TEKread(fptek);
    
    VERBOSE (printf("Generating the switches\n"));
    LIBfillAll();    /* read the layout information from library */
    
    VERBOSE(printf("Evaluating the capacitor ratios and unit capacitor areas\n"));
    CBLKcapRatios();	/* some checking, no other routines called */
    CBLKucapArea();	/* some checking, no other routines called */
    
    
    VERBOSE(printf("Evaluating the dimensions of capacitor blocks\n"));
    CBLKfindDim();
    
    VERBOSE(printf("Ordering the capacitor blocks\n"));
    PERMcblk(0., 1000);

    VERBOSE(printf("Initial ordering of the opamps\n"));
    PERMmodules( OPAMP, APPROX);

    VERBOSE(printf("Initial ordering of the switches\n"));
    PERMmodules( SWCH, APPROX);

    for ( counter = 1; counter <= 1; counter++ ) { 
	VERBOSE(printf("Ordering the capacitors \n"));
	PERMallCaps();
	
	VERBOSE(printf("Generating symbolic information for capacitor blocks\n"));
	for ( tmpmod = modlist[CBLK]; tmpmod != NULL; tmpmod = tmpmod->next ) {
	    tmpcblk = tmpmod->info.cblk;
	    CBLKgenSymb(tmpcblk);
	    CBLKgenTerms( tmpcblk, NO, instances );
	    CBLKfindEqt( tmpcblk, NO );
	}
	
	/* reorder the opamps and switches with exact cblk terminal locations */
	VERBOSE(printf("Reordering the opamps\n"));
	PERMmodules( OPAMP, EXACT);
	VERBOSE(printf("Reordering the switches\n"));
	PERMmodules( SWCH, EXACT);
    }
    
    VERBOSE(printf("Generating layouts for capacitor blocks\n"));
    for ( tmpmod = modlist[CBLK]; tmpmod != NULL; tmpmod = tmpmod->next ) {
	tmpcblk = tmpmod->info.cblk;
	CBLKgenSymb(tmpcblk);
	/* the layout generation should be done before equivalent terminal
	   location so the location of HNUC and VNUC is known */
	DEBUG (printf ("finished CBLKgenSymb...\n"));
	CBLKsymb2lay( tmpcblk, instances );
	DEBUG (printf ("finished CBLKsymb2lay...\n"));
	CBLKgenTerms( tmpcblk, YES, instances );
	/* if the capacitors are not reorderd we don't need to repeat this */
	DEBUG (printf ("finished  CBLKgenTerms...\n"));
	CBLKfindEqt( tmpcblk, YES );
	DEBUG (printf ("finished CBLKfindEqt...\n"));
	CBLKtermLoc( tmpcblk );
    }

    VERBOSE (printf("Defining the routing channels\n")); 
    ROUTmkTlist();

    VERBOSE(printf("Routing the switch channel\n"));
    ROUTdoAll( chanlist[TOP],tmp_dir,instances );

    VERBOSE(printf("Routing the op-amp channel\n"));
    ROUTdoAll( chanlist[BOTTOM],tmp_dir,instances );

    VERBOSE(printf("Routing the clock channel\n"));
    ROUTdoAll( chanlist[CLKCH],tmp_dir,instances );

    ADclose (fpckt, fptek, fpout, tmp_dir, T_flg);
    
    VERBOSE (printf ("Creating the output symbolic cell...\n"));
    ADsymbolic(outputcell, lib, instances, noClockLines, 1);

    VERBOSE (printf("Execution completed\n"));
    exit (0);
}
