/*
 * Copyright (c) 1985 Thomas L. Quarles
 */
 /* last change: 6/30/87 by Hormoz Yaghutiel */
#ifndef LINT
static char rcsid[] =
    "INPpass2.c $Revision: 7.1 $ on $Date: 86/03/31 11:51:14 $";
#endif LINT

/*
 * Modifications to Adore
 * Henry Chang (henryc@ic.Berkeley.EDU)
 * Fall, 1990
 */

#define C_FILE_NAME "INPpass2.c"

#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "utility.h"
#include "INPdefs.h"
#include "ADglobals.h"

typedef union PARmValue {
    int ivalue;
    double rvalue;
    struct v {  
        double *vvalue;
        int nvalue;
    } v;
    char * namValue;
} PARmValue;

    /* pass 2 - pass through the card images built up so far and
     *  perform the appropriate actions.  We have pre-read all of
     *  the .MODEL cards, so we can ignore them
     */

INPchkLibName (lib,libname)
char *lib;
char *libname;
/*
 * Routine that will checking in the directory specified with the -L option
 * -hcc
 */
{
    char *i;
    struct stat buf;                    /* Used for stat command */
    char tmplibname[FILENAME_SIZE];

    /*
     * first check if there is a "/" in it.  If there is, then the -L
     * option value will be ignored.
     */
    for (i=libname;*i!=0;i++)
       if (*i=='/') return 0;

    /*
     * Check for existance of libname in the current working directory.
     * If it exists, then option value is ignored.
     */
    if ( stat(libname,&buf) == 0 ) {
       return 0;
    }

    /*
     * Otherwise tack on value in library option.
     */
    strcpy (tmplibname,lib);
    strcat (tmplibname,"/");
    strcat (tmplibname,libname);
    strcpy (libname,tmplibname);
} /* INPchkLibName */


void 
INPpass2(data,lib) 
card *data;
char *lib;
{

  int length;
  card *current;
  char *token;
  char *line;
  char *node1,*node2,*node3,*node4;
  char *amp1term, *clk1term, *capterm, *clk2term, *amp2term;
  struct _net  *netptr1,*netptr2,*netptr3,*netptr4,*netptr5;
  struct _net *INPtermInsert();
  struct _lib *INPlibInsert();
  struct _lib *libptr;
  char *name, tmpname[40];
  char c;
  char *libname;			
  char *full_libname;		         /* libname with -L information */
  char *clkname;
  char *temp;
  int type;
  PARmValue ptemp;
  char * groundname="0";
  char * gname;
  double INPevaluate(), val;
  char *word;
  int error;


  for (current = data, error = 0; current; current = current->nextcard) {
    error++;
  }
  INPtabInit(error);
  INPgetTok(&groundname,&gname);
  netptr1 = INPtermInsert(&gname);
  netptr1->type = GROUND;

  /* allocate memory for dummy input and output modules */
  INPcrtIOmod();

  for(current = data; current != NULL; current = current->nextcard) {
    c = *(current->line);
    c = islower(c) ? toupper(c) : c;
    if(c) switch(c) {

    case ' ':			/* blank line (space leading) */
    case '\t':			/* blank line (tab leading) */
      break;

    case 'C':			/* Cname <node> <node> <val>  */
      line = current->line;
      INPgetTok(&line,&name);

      INPgetTok(&line,&node1);
      netptr1 = INPtermInsert(&node1);

      INPgetTok(&line,&node2);
      netptr2 = INPtermInsert(&node2);

      val = INPevaluate(&line,&error);
      if(error == 0)  {		/* Looks like a number */
	ptemp.rvalue = val;
      }  else  {		/* looks like character strings */
	/* this is an error */
      }

      /* create storage for the capacitor */
      INPcrtCap(name, netptr1, netptr2, val);
      break;
    case 'E':		/* Ename <node> <node> <node> <node> <libname> */
      line = current->line;
      INPgetTok(&line,&name);

      INPgetTok(&line,&node1);
      netptr1 = INPtermInsert(&node1);

      INPgetTok(&line,&node2);
      netptr2 = INPtermInsert(&node2);

      INPgetTok(&line,&node3);
      netptr3 = INPtermInsert(&node3);

      INPgetTok(&line,&node4);
      netptr4 = INPtermInsert(&node4);

      INPgetTok(&line,&libname);
      /*
       * libname is a pointer to the library name.
       * it has been only MALLOCed to the size of the library name.
       * We need to insert this into a symbol table with the path name,
       * therefore we needed something that is MALLOCed larger.
       * We will free libname, and MALLOC or larger pointer.
       * -hcc
       */
      MALLOC_ERRCHK (full_libname, char, FILENAME_SIZE);
      strcpy (full_libname,libname);
      FREE(libname);
      INPchkLibName (lib,full_libname);
      libptr = INPlibInsert(&full_libname);

      INPcrtInteg(name, netptr1, netptr2, netptr3, netptr4, libptr);
      break;
      /* output definitions */
    case 'O':			/* Oname <node> <node> */
      line = current->line;
      INPgetTok(&line,&name);

      INPgetTok(&line,&node1);
      netptr1 = INPtermInsert(&node1);

      INPgetTok(&line,&node2);
      netptr2 = INPtermInsert(&node2);
      if ( netptr1->type != GROUND ) {
	sprintf(tmpname, "%s%s", name, "+");
	INPcrtIOterm(tmpname, netptr1, OUTMOD);
      }
      if ( netptr2->type != GROUND ) {
	sprintf(tmpname, "%s%s", name, "-");
	INPcrtIOterm(tmpname, netptr2, OUTMOD);
      }
      break;
    case 'V':			/* Vname <node> <node> */
      line = current->line;
      INPgetTok(&line,&name);

      INPgetTok(&line,&node1);
      netptr1 = INPtermInsert(&node1);

      INPgetTok(&line,&node2);
      netptr2 = INPtermInsert(&node2);
      if ( netptr1->type != GROUND ) {
	sprintf(tmpname, "%s%s", name, "+");
	INPcrtIOterm(tmpname, netptr1, INMOD);
      }
      if ( netptr2->type != GROUND ) {
	sprintf(tmpname, "%s%s", name, "-");
	INPcrtIOterm(tmpname, netptr2, INMOD);
      }
      break;
      /*  VOLTAGE CONTROLLED SWITCH */
    case 'S':	/* Sname <amp1node> <clk1> <capnode> <clk2> <amp2node> */
      line = current->line;
      INPgetTok(&line,&name);

      INPgetTok(&line,&amp1term);
      netptr1 = INPtermInsert(&amp1term);

      INPgetTok(&line,&clk1term);
      netptr2 = INPtermInsert(&clk1term);

      INPgetTok(&line,&capterm);
      netptr3 = INPtermInsert(&capterm);

      INPgetTok(&line,&clk2term);
      netptr4 = INPtermInsert(&clk2term);

      INPgetTok(&line,&amp2term);
      netptr5 = INPtermInsert(&amp2term);

      INPgetTok(&line,&libname);
      if ( strcmp(libname,"") == 0 ) {
	/* No library name specified */
	libptr = NULL;
      }
      else {
	libptr = INPlibInsert(&libname);
      }

      INPcrtSw(name, netptr1, netptr2, netptr3, netptr4, netptr5, libptr);
      break;

    case '*':			/* *<anything> - a comment - ignore */
      break;

    default:
      /* the un-implemented device */
      temp = ALLOC(char, 100);
      (void) strcpy(temp," unknown device type - error \n");
      current->error = INPerrCat(current->error,temp);
      break;
    }
  }
 end:
  /**** do not destroy the table structure, it is neede later */
  /*
    INPtabEnd();
    */
  return;
}

