/* This procedure evluates the maximum density of a channel for adore2 */
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni-Vincentelli */
/* Last Change : 8/10/87 */

#include "ADglobals.h"
#include <stdio.h>

/* First for each net the pins with minimum and maximum x-coordinate
are marked. Then starting from the first column, everytime we encounter
a pin with minimum x-coordinate for its net, we increase the channel
density of that column and the columns there-after. Every time we
encounter a pin with maximum x-coordinate for its net, we decrease
the density of the next column and the columns there-after.
Maximum density of all columns is recorded. */

ROUTchDens( channel )
struct _channel *channel;
{
	int counter;
	int chminx, chmaxx;
	int termx;
	int density;

	struct _net *tmpnet;
	struct _terminal *tmpterm;


	/*** first find the limits of each net in the channel */

	channel->numnets = 0;
	for ( tmpnet = netlist; tmpnet != NULL; tmpnet = tmpnet->next ) {
		if (tmpnet->type == GROUND ) {
			continue;
		}
		chmaxx = - (MAXINT - 1);
		chminx = MAXINT;
		tmpterm = tmpnet->firstnett; 
		for ( ; tmpterm != NULL; tmpterm = tmpterm->nxtnett ) {
			if ( tmpterm->channel == channel ) {
				if ( tmpterm->module->type == OUTMOD ) {
					termx = MAXINT;
				}
				else {
					termx = PERMfindXcrd(tmpterm);
				}
				if ( termx < chminx ) {
					chminx = termx;
					tmpnet->chminxt[channel->name] = tmpterm;
				}
				if ( termx > chmaxx ) {
					chmaxx = termx;
					tmpnet->chmaxxt[channel->name] = tmpterm;
				}
			}
		}
		if ( chminx != MAXINT ) {
			/*** there is at least one terminal of this net in the channel */
			channel->numnets++;
		}
	}

	density = channel->maxdensity = channel->numterms[LEFT];
	for ( counter =channel->lowchx +1; counter <=channel->hichx -1; counter++ ){
		tmpterm = channel->xgrids[counter].topterm;
		if ( tmpterm != NULL ) { 
			if ( tmpterm->netptr->chminxt[channel->name] == tmpterm) {
				density++;
			}
			if ( tmpterm->netptr->chmaxxt[channel->name] == tmpterm) {
				density--;
			}
		}

		tmpterm = channel->xgrids[counter].botterm;
		if ( tmpterm != NULL ) { 
			if ( tmpterm->netptr->chminxt[channel->name] == tmpterm) {
				density++;
			}
			if ( tmpterm->netptr->chmaxxt[channel->name] == tmpterm) {
				density--;
			}
		}

		if ( density > channel->maxdensity ) {
			channel->maxdensity = density;
		}
	}
	if ( density != channel->numterms[RIGHT] ) {
		fprintf(stderr,"ROUTchDens;1: internal error\n");
		exit(-1);
	}
}
