/*    adTekRead.c
 *
 *    Author: Enrico Malavasi  -  March 1992
 *
 *    Purpose: read the technology information for Adore2. This is
 *         a standard technology access routine. It should substitute
 *         in this TEKread.c. No more information should be provided
 *         by the tekfile than eventually the technology name and a few options.
 */
#define C_FILE_NAME "adTekRead.c"

#include <stdio.h>
#include "oct.h"
#include "oh.h"
#include "utility.h"
#include "tap.h"
#include "th.h"

int metmetsp,                   /* spacing betw. adjacent metal lines */
    metwidth,
    polypolysp,                 /* Spacing betw. poly and poly */
    polywidth,
    polyndifsp,
    polypdifsp,
    metovcont,
    polyovcon,
    polyconsp,
    conconsp,                   /*  Spacing betw. adjacent contacts  */
    pwelpdifsp,                 /* pwell and pdiffusion spacing */
    pwelpwelsp,                 /* spacing between two pwells */
    nwelnwelsp,                 /* spacing between two nwells */
    pwelovpdif,
    pwelovndif,
    ndifovcon,
    pdifovcon;

int mincontd,           /*  Minimum contact dimension */
    minctctsp;                  /*minimum center to center via spacing */

char  *botpoly_L,
      *pwel_L,
      *nwel_L,
      *pdif_L,
      *ndif_L,
      *con_L,
      *toppoly_L,
      *metal_L,
      *select_L,
      *cop2_L,
      *technName;               /*  technology name  */

#define   BOTPOLY_LDEF  "POLY"
#define   PWEL_LDEF     "PWEL"
#define   NWEL_LDEF     "NWEL"
#define   PDIF_LDEF     "PDIF"
#define   NDIF_LDEF     "NDIF"
#define   METAL_LDEF    "MET1"
#define   CON_LDEF      "COPO"
#define   TOPPOLY_LDEF  "POLY2"
#define   COP2_LDEF     "COP2"
#define   SELECT_LDEF   "NONDEF"
#define   TECHNNAME_DEF "scmos"

adTekRead()
{
    octObject   dummyFacet;
    octObject   metalLayer;
    octObject   polyLayer;
    octObject   ndifLayer;
    octObject   pdifLayer;
    octObject   pwelLayer;
    octObject   nwelLayer;
    octObject   condLayer;
    octObject   copdLayer;
    octObject   copoLayer;
    char        dummyName[256];

    botpoly_L = BOTPOLY_LDEF;
    pwel_L    =    PWEL_LDEF;
    nwel_L    =    NWEL_LDEF;
    pdif_L    =    PDIF_LDEF;
    ndif_L    =    NDIF_LDEF;
    metal_L   =   METAL_LDEF;
    con_L     =     CON_LDEF;
    toppoly_L = TOPPOLY_LDEF;
    cop2_L    =    COP2_LDEF;
    select_L  =  SELECT_LDEF;
    technName = TECHNNAME_DEF;

    sprintf(dummyName, "$OCTTOOLS/tech/%s/tap.views", technName);
    OH_ASSERT(ohOpenFacet(&dummyFacet, dummyName, "symbolic", "contents", "r"));
    
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &metalLayer, metal_L));
    metmetsp   = tapGetMinSpacing(&metalLayer, &metalLayer);
    metwidth   = tapGetMinWidth(&metalLayer);
    
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &polyLayer, botpoly_L));
    polypolysp = tapGetMinSpacing(&polyLayer, &polyLayer);
    polywidth  = tapGetMinWidth(&polyLayer);
    
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &ndifLayer, ndif_L));
    polyndifsp = tapGetMinSpacing(&polyLayer, &ndifLayer);
    
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &pdifLayer, pdif_L));
    polypdifsp = tapGetMinSpacing(&polyLayer, &pdifLayer);
    
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &nwelLayer, nwel_L));
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &pwelLayer, pwel_L));
    pwelpdifsp = tapGetMinSpacing(&pwelLayer, &pdifLayer);
    pwelpwelsp = tapGetMinSpacing(&pwelLayer, &pwelLayer);
    pwelovndif = tapGetMinOverlap(&pwelLayer, &ndifLayer);
    pwelovpdif = tapGetMinOverlap(&pwelLayer, &pdifLayer);

    nwelnwelsp = tapGetMinSpacing(&nwelLayer, &nwelLayer);

    OH_ASSERT(ohGetByLayerName(&dummyFacet, &copoLayer, con_L));
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &condLayer, "COND"));
    OH_ASSERT(ohGetByLayerName(&dummyFacet, &copdLayer, "COPD"));
    metovcont  =  tapGetMinOverlap(&metalLayer, &copoLayer);
    conconsp   =  tapGetMinSpacing(&copoLayer, &copoLayer);
    polyovcon  =  tapGetMinOverlap(&polyLayer, &copoLayer);
    ndifovcon  =  tapGetMinOverlap(&ndifLayer, &condLayer);
    pdifovcon  =  tapGetMinOverlap(&pdifLayer, &copdLayer);
    mincontd   =  tapGetMinWidth( &copoLayer );
    
    OH_ASSERT(octCloseFacet(&dummyFacet));
}
