/* Isolate the capacitor blocks from the substrate by placing a pwell */ 
/* Author: Hormoz Yaghutiel, Advisor: Alberto Sangiovanni Vincentelli */
/* Last Change: 9/8/87 */

#include <stdio.h>
#include "ADglobals.h"
#include "TEKtech.h"
#include "oct.h"
#include "symlib.h"
#include "oh.h"

#define    WELL_OVERHANG   200   /* Overhang of well beyond capacitor blocks  */
#define    WELL_TERMWIDTH  200   /* Well terminal width  */

adWells( facet, wellType, wellBias, instanceDir )
octObject *facet;
char       wellType;
char      *wellBias;
char      *instanceDir;
{
    char      *wellLayerName;
    octObject  instance;
    octObject  layer;
    octObject  term1, term2;
    struct octPoint pt1, pt2;
    struct _module *mod, *tmpmod;
    octCoord  wWidth, x, wHeight, halfHeight;
    octCoord  wellOverhang = WELL_OVERHANG;

    int lowleftx, lowlefty, uprightx, uprighty;
    int slowleftx, slowlefty, suprightx, suprighty;
    int buswidth;
    int miny;
    struct _bus *tmpbus, *vggbus;

    wellLayerName = (wellType == 'n') ? "NWEL" : "PWEL";

    halfHeight = (modlist[CBLK]->ydim + 2*wellOverhang)/2;   LAMBDA_ROUND(halfHeight);
    wHeight = halfHeight + halfHeight;
    LAMBDA_ROUND( wellOverhang );

    ADSsquareConn( facet, &instance, wellLayerName, WELL_TERMWIDTH, wHeight, instanceDir );
    instance.contents.instance.name = "leftWellSide";
    instance.contents.instance.transform.translation.x = modlist[CBLK]->xtrans - wellOverhang;
    instance.contents.instance.transform.translation.y = modlist[CBLK]->ytrans +halfHeight-wellOverhang;
    octModify( &instance );
    OH_ASSERT( octGenFirstContent( &instance, OCT_TERM_MASK, &term1 ) );

    /*  Look for the rightmost cap block */
    mod = modlist[CBLK];
    for(tmpmod=mod->next ; tmpmod!=NULL ; tmpmod=tmpmod->next) {
	if( tmpmod->xtrans > mod->xtrans )  mod = tmpmod;
    }

    instance.contents.instance.name = "rightWellSide";
    instance.contents.instance.transform.translation.x = mod->xtrans + mod->xdim + wellOverhang;
    instance.contents.instance.transform.translation.y = modlist[CBLK]->ytrans + halfHeight-wellOverhang;
    octCreate( facet, &instance );
    OH_ASSERT( octGenFirstContent( &instance, OCT_TERM_MASK, &term2 ) );

    ohGetOrCreateLayer( facet, &layer, wellLayerName );
    symCreateSegment( &layer,
		     &pt1, &term1, SYM_USE_EXISTING,
		     &pt2, &term2, SYM_USE_EXISTING,
		     wHeight, SYM_HORIZONTAL );

    return;

    /* put the p-well under the capacitors */
    lowleftx = 0;
    lowlefty = chanlist[BOTTOM]->module->ytrans + pwelpwelsp;
    uprightx = lowleftx + chipxdim;
    uprighty = modlist[SWCH]->ytrans - pwelpwelsp;
    suprighty = uprighty  - pwelovpdif;
    makebox(facet, pwel_L,lowleftx, lowlefty, uprightx, uprighty);
    
    lowleftx = uprightx;
    lowlefty = modlist[CBLK]->ytrans;
    slowlefty = lowlefty + pwelovpdif;
    uprightx = lowleftx + 2 * metmetsp + buswidth; 
    makebox(facet,pwel_L,lowleftx, lowlefty, uprightx, uprighty);
    
    slowleftx = lowleftx += 2 * metmetsp;
    uprightx = lowleftx + buswidth;
    uprighty = vggbus->uprighty + modlist[SWCH]->ytrans;
    makebox(facet,metal_L,lowleftx, lowlefty, uprightx, uprighty);
    
    uprightx = lowleftx;
    lowleftx = chipxdim;
    lowlefty = vggbus->lowlefty + modlist[SWCH]->ytrans;
    uprighty = vggbus->uprighty + modlist[SWCH]->ytrans;
    makebox(facet,metal_L,lowleftx, lowlefty, uprightx, uprighty);
    
    /* now make the contacts from the well to vss */
    lowleftx = slowleftx + (buswidth - mincontd)/2;
    lowlefty = slowlefty + mincontd/2 + pdifovcon;
    uprightx = lowleftx + mincontd;
    uprighty = lowlefty + mincontd;
    while ( uprighty <= (suprighty - pdifovcon - mincontd/2) ) {
	makebox(facet,con_L,lowleftx, lowlefty, uprightx, uprighty);
	lowlefty += mincontd + conconsp;
	uprighty = lowlefty + mincontd;
    }
    lowleftx -= pdifovcon;
    lowlefty = slowlefty;
    uprightx += pdifovcon;
    uprighty = suprighty;
    makebox(facet,pdif_L,lowleftx, lowlefty, uprightx, uprighty);
    
    lowleftx -= selovpdif;
    lowlefty -= selovpdif;
    uprightx += selovpdif;
    uprighty += selovpdif;
    makebox(facet,select_L,lowleftx, lowlefty, uprightx, uprighty);

    octCloseFacet( facet );
}
